@@ -5,7 +5,7 @@
  * Copyright (c) 2000-2004 Ethan Galstad (nagios@nagios.org)
  * Copyright (c) 2007 nagios-plugins team
  * License: GPL
- * Last Modified: $Date: 2007/05/12 07:45:36 $
+ * Last Modified: $Date: 2007-06-20 10:01:07 +0100 (Wed, 20 Jun 2007) $
  *
  * License Information:
  *
@@ -23,12 +23,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: check_cluster.c,v 1.7 2007/05/12 07:45:36 psychotrahe Exp $
+ * $Id: check_cluster.c 1745 2007-06-20 09:01:07Z psychotrahe $
  * 
 ******************************************************************************/
 
 const char *progname = "check_cluster";
-const char *revision = "$Revision: 1.7 $";
+const char *revision = "$Revision: 1745 $";
 const char *copyright = "2007";
 const char *email = "nagiosplug-devel@lists.sourceforge.net";
 
@@ -151,6 +151,7 @@ int process_arguments(int argc, char **argv){
 		{"host",     no_argument,      0,'h'},
 		{"service",  no_argument,      0,'s'},
 		{"verbose",  no_argument,      0,'v'},
+		{"version",  no_argument,      0,'V'},
 		{"help",     no_argument,      0,'H'},
 		{0,0,0,0}
 	};
@@ -161,7 +162,7 @@ int process_arguments(int argc, char **argv){
 
 	while(1){
 
-		c=getopt_long(argc,argv,"hHsvw:c:d:l:",longopts,&option);
+		c=getopt_long(argc,argv,"hHsvVw:c:d:l:",longopts,&option);
 
 		if(c==-1 || c==EOF || c==1)
 			break;
@@ -200,6 +201,11 @@ int process_arguments(int argc, char **argv){
 			verbose++;
 			break;
 
+		case 'V': /* version */
+			print_revision (progname, revision);
+			exit (STATE_OK);
+			break;
+
 		case 'H': /* help */
 			print_help();
 			exit(STATE_UNKNOWN);
